"use strict";
// Copyright (c) Bloomberg
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.HyperlinkRendererView = exports.HyperlinkRendererModel = exports.BarRendererView = exports.BarRendererModel = exports.TextRendererView = exports.TextRendererModel = exports.CellRendererView = exports.CellRendererModel = void 0;
const d3Format = require('d3-format');
const d3TimeFormat = require('d3-time-format');
const datagrid_1 = require("@lumino/datagrid");
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
const barrenderer_1 = require("./core/barrenderer");
const vegaexpr_1 = require("./vegaexpr");
const utils_1 = require("./utils");
class CellRendererModel extends base_1.WidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: CellRendererModel.model_name, _model_module: CellRendererModel.model_module, _model_module_version: CellRendererModel.model_module_version, _view_name: CellRendererModel.view_name, _view_module: CellRendererModel.view_module, _view_module_version: CellRendererModel.view_module_version });
    }
}
exports.CellRendererModel = CellRendererModel;
CellRendererModel.model_name = 'CellRendererModel';
CellRendererModel.model_module = version_1.MODULE_NAME;
CellRendererModel.model_module_version = version_1.MODULE_VERSION;
CellRendererModel.view_name = 'CellRendererView';
CellRendererModel.view_module = version_1.MODULE_NAME;
CellRendererModel.view_module_version = version_1.MODULE_VERSION;
class CellRendererView extends base_1.WidgetView {
    constructor() {
        super(...arguments);
        this.processors = {};
    }
    render() {
        return this.initializeRenderer().then(() => {
            this.updateRenderer();
            // @ts-ignore
            this.on('renderer-needs-update', this.updateRenderer.bind(this));
        });
    }
    /**
     * Method that should be called when the theme has changed.
     */
    onThemeChanged() {
        return this.initializeRenderer().then(() => {
            this.updateRenderer();
        });
    }
    /**
     * Initialize the CellRenderer widget.
     *
     * @return The promise to initialize the renderer
     */
    initializeRenderer() {
        const promises = {};
        const attr_names = this.model
            .get_attrs()
            .map((attr) => {
            return attr.name;
        });
        this.model.on_some_change(attr_names, this._on_some_processors_change, this);
        for (const name of attr_names) {
            promises[name] = this.updateProcessor(name);
        }
        return (0, base_1.resolvePromisesDict)(promises).then((processors) => {
            this.processors = processors;
        });
    }
    _on_some_processors_change(event) {
        const promises = {};
        for (const name in event.changed) {
            promises[name] = this.updateProcessor(name);
        }
        (0, base_1.resolvePromisesDict)(promises).then((processors) => {
            this.processors = Object.assign(Object.assign({}, this.processors), processors);
            // @ts-ignore
            this.trigger('renderer-needs-update');
        });
    }
    /**
     * Update the phosphor renderer value, and trigger an event so that the DataGrid widget knows it has
     * changed.
     */
    updateRenderer() {
        const options = {};
        for (const attr of this.model.get_attrs()) {
            if (attr.phosphorName) {
                options[attr.phosphorName] = (config) => {
                    return this.process(attr.name, config, attr.defaultValue);
                };
            }
        }
        this.renderer = this.createRenderer(options);
        // @ts-ignore
        this.trigger('renderer-changed');
    }
    /**
     * Update the processor associated with the given name.
     *
     * @param name - The name of the attribute to process.
     *
     * @return The PromiseLike to update the processor view.
     */
    updateProcessor(name) {
        // @ts-ignore
        const processor = this.model.get(name);
        if (utils_1.Scalar.isScalar(processor)) {
            return processor;
        }
        // Assuming it is an VegaExprModel or a Scale model
        // @ts-ignore
        this.listenTo(processor, 'change', () => {
            // @ts-ignore
            this.trigger('renderer-needs-update');
        });
        return this.create_child_view(processor);
    }
    /**
     * Process a cell attribute given the cell config.
     *
     * @param name - The name of the attribute to process.
     *
     * @param config - The configuration data for the cell.
     *
     * @param defaultValue - The default attribute value.
     */
    process(name, config, defaultValue) {
        const processor = this.processors[name];
        if (utils_1.Scalar.isScalar(processor)) {
            if (name === 'font' &&
                typeof processor === 'string' &&
                ((typeof config.value === 'number' && !Number.isFinite(config.value)) ||
                    (config.value instanceof Date &&
                        Number.isNaN(config.value.getTime())))) {
                return `italic ${processor}`;
            }
            return processor;
        }
        if (processor instanceof vegaexpr_1.VegaExprView) {
            return processor.process(config, defaultValue);
        }
        // If it's a DateScale, convert the value to a Date object
        if (processor.model.type == 'date' ||
            processor.model.type == 'date_color_linear') {
            return processor.scale(new Date(config.value));
        }
        // Assuming it is a Scale view
        return processor.scale(config.value);
    }
}
exports.CellRendererView = CellRendererView;
class TextRendererModel extends CellRendererModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: TextRendererModel.model_name, _view_name: TextRendererModel.view_name, font: '12px sans-serif', text_color: null, text_wrap: false, text_elide_direction: 'right', text_value: null, background_color: null, vertical_alignment: 'center', horizontal_alignment: 'left', format: null, format_type: 'number', missing: '' });
    }
    get_attrs() {
        return [
            { name: 'font', phosphorName: 'font', defaultValue: '12px sans-serif' },
            { name: 'text_wrap', phosphorName: 'wrapText', defaultValue: false },
            {
                name: 'text_elide_direction',
                phosphorName: 'elideDirection',
                defaultValue: 'right',
            },
            {
                name: 'text_color',
                phosphorName: 'textColor',
                defaultValue: utils_1.Theme.getFontColor(),
            },
            { name: 'text_value', phosphorName: null, defaultValue: null },
            {
                name: 'background_color',
                phosphorName: 'backgroundColor',
                defaultValue: utils_1.Theme.getBackgroundColor(),
            },
            {
                name: 'vertical_alignment',
                phosphorName: 'verticalAlignment',
                defaultValue: 'center',
            },
            {
                name: 'horizontal_alignment',
                phosphorName: 'horizontalAlignment',
                defaultValue: 'left',
            },
            { name: 'format', phosphorName: null, defaultValue: null },
        ];
    }
}
exports.TextRendererModel = TextRendererModel;
TextRendererModel.serializers = Object.assign(Object.assign({}, CellRendererModel.serializers), { font: { deserialize: base_1.unpack_models }, text_color: { deserialize: base_1.unpack_models }, text_wrap: { deserialize: base_1.unpack_models }, text_elide_direction: { deserialize: base_1.unpack_models }, text_value: { deserialize: base_1.unpack_models }, background_color: { deserialize: base_1.unpack_models }, vertical_alignment: { deserialize: base_1.unpack_models }, horizontal_alignment: { deserialize: base_1.unpack_models }, format: { deserialize: base_1.unpack_models } });
TextRendererModel.model_name = 'TextRendererModel';
TextRendererModel.view_name = 'TextRendererView';
class TextRendererView extends CellRendererView {
    render() {
        return super.render().then(() => {
            this.model.on_some_change(['missing', 'format_type'], () => {
                // @ts-ignore
                this.trigger('renderer-needs-update');
            }, this);
        });
    }
    createRenderer(options) {
        return new datagrid_1.TextRenderer(Object.assign(Object.assign({}, options), { format: this.getFormatter() }));
    }
    getFormatter(options = {}) {
        return (config) => {
            let formattedValue;
            if (config.value === null) {
                // @ts-ignore
                formattedValue = this.model.get('missing');
            }
            else {
                const formattingRule = this.process('format', config, null);
                if (formattingRule === null) {
                    formattedValue = String(config.value);
                }
                else {
                    // @ts-ignore
                    if (this.model.get('format_type') == 'time') {
                        formattedValue = String(d3TimeFormat.timeFormat(formattingRule)(new Date(config.value)));
                    }
                    else {
                        formattedValue = String(d3Format.format(formattingRule)(config.value));
                    }
                }
            }
            return (this.process('text_value', config, formattedValue) || formattedValue);
        };
    }
}
exports.TextRendererView = TextRendererView;
class BarRendererModel extends TextRendererModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: BarRendererModel.model_name, _view_name: BarRendererModel.view_name, bar_color: '#4682b4', bar_value: 0, orientation: 'horizontal', bar_vertical_alignment: 'bottom', bar_horizontal_alignment: 'left', show_text: true });
    }
    get_attrs() {
        return super.get_attrs().concat([
            { name: 'bar_color', phosphorName: 'barColor', defaultValue: '#4682b4' },
            { name: 'bar_value', phosphorName: 'barValue', defaultValue: 0 },
            {
                name: 'orientation',
                phosphorName: 'orientation',
                defaultValue: 'horizontal',
            },
            {
                name: 'bar_vertical_alignment',
                phosphorName: 'barVerticalAlignment',
                defaultValue: 'bottom',
            },
            {
                name: 'bar_horizontal_alignment',
                phosphorName: 'barHorizontalAlignment',
                defaultValue: 'left',
            },
            { name: 'show_text', phosphorName: 'showText', defaultValue: true },
        ]);
    }
}
exports.BarRendererModel = BarRendererModel;
BarRendererModel.serializers = Object.assign(Object.assign({}, TextRendererModel.serializers), { bar_color: { deserialize: base_1.unpack_models }, bar_value: { deserialize: base_1.unpack_models }, orientation: { deserialize: base_1.unpack_models }, bar_vertical_alignment: { deserialize: base_1.unpack_models }, bar_horizontal_alignment: { deserialize: base_1.unpack_models }, show_text: { deserialize: base_1.unpack_models } });
BarRendererModel.model_name = 'BarRendererModel';
BarRendererModel.view_name = 'BarRendererView';
class BarRendererView extends TextRendererView {
    createRenderer(options) {
        return new barrenderer_1.BarRenderer(Object.assign(Object.assign({}, options), { format: this.getFormatter() }));
    }
}
exports.BarRendererView = BarRendererView;
class HyperlinkRendererModel extends TextRendererModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: BarRendererModel.model_name, _view_name: BarRendererModel.view_name, url: {}, url_name: {} });
    }
    get_attrs() {
        return super.get_attrs().concat([
            { name: 'url', phosphorName: 'url', defaultValue: null },
            { name: 'url_name', phosphorName: 'urlName', defaultValue: null },
        ]);
    }
}
exports.HyperlinkRendererModel = HyperlinkRendererModel;
HyperlinkRendererModel.serializers = Object.assign(Object.assign({}, TextRendererModel.serializers), { url: { deserialize: base_1.unpack_models }, url_name: { deserialize: base_1.unpack_models } });
HyperlinkRendererModel.model_name = 'HyperlinkRendererModel';
HyperlinkRendererModel.view_name = 'HyperlinkRendererView';
class HyperlinkRendererView extends TextRendererView {
    createRenderer(options) {
        return new datagrid_1.HyperlinkRenderer(Object.assign(Object.assign({}, options), { format: this.getFormatter() }));
    }
}
exports.HyperlinkRendererView = HyperlinkRendererView;
//# sourceMappingURL=cellrenderer.js.map