import { ReadonlyJSONObject } from '@lumino/coreutils';
import { DataModel } from '@lumino/datagrid';
import { ViewBasedJSONModel } from './viewbasedjsonmodel';
/**
 * A View implementation for immutable in-memory JSON data.
 *
 * Note: Most of this is just repurposed from JSONModel, and can likely be
 * streamlined quite a bit.
 */
export declare class View {
    /**
     * Create a view with static JSON data.
     *
     * @param options - The options for initializing the view.
     */
    constructor(options: View.IOptions);
    /**
     * Get the row count for a region in the view.
     *
     * @param region - The row region of interest.
     *
     * @returns - The row count for the region.
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * Get the column count for a region in the view.
     *
     * @param region - The column region of interest.
     *
     * @returns - The column count for the region.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * Get the metadata for a column in the view.
     *
     * @param region - The cell region of interest.
     *
     * @param column - The index of the column of interest.
     *
     * @returns The metadata for the column.
     */
    metadata(region: DataModel.CellRegion, row: number, column: number): DataModel.Metadata;
    /**
     * Get the data value for a cell in the view.
     *
     * @param region - The cell region of interest.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param column - The column index of the cell of interest.
     *
     * @param returns - The data value for the specified cell.
     *
     * #### Notes
     * A `missingValue` as defined by the schema is converted to `null`.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * Returns a reference to the dataset from this View.
     */
    get dataset(): View.DataSource;
    /**
     * Returns the index in the schema that relates to the index by region.
     *
     * @param region - The `CellRegion` of interest.
     *
     * @param index - The column index to look up.
     */
    getSchemaIndex(region: DataModel.CellRegion, index: number): number;
    /**
     * Returns a Promise that resolves to an array of unique values contained in
     * the provided column index.
     *
     * @param columnIndex - The index to retrieve unique values for.
     */
    uniqueValues(region: DataModel.CellRegion, columnIndex: number): Promise<any[]>;
    private readonly _data;
    private readonly _bodyFields;
    private readonly _headerFields;
    private readonly _missingValues;
}
/**
 * The namespace for the `View` class statics.
 */
export declare namespace View {
    /**
     * An object which describes a column of data in the view.
     *
     * #### Notes
     * This is based on the JSON Table Schema specification:
     * https://specs.frictionlessdata.io/table-schema/
     */
    /**
     * An object when specifies the schema for a view.
     *
     * #### Notes
     * This is based on the JSON Table Schema specification:
     * https://specs.frictionlessdata.io/table-schema/
     */
    interface ISchema {
        /**
         * The fields which describe the view columns.
         *
         * Primary key fields are rendered as row header columns.
         */
        readonly fields: ViewBasedJSONModel.IField[];
        /**
         * The values to treat as "missing" data.
         *
         * Missing values are automatically converted to `null`.
         */
        readonly missingValues?: string[];
        /**
         * The field names which act as primary keys.
         *
         * Primary key fields are rendered as row header columns.
         */
        readonly primaryKey?: string | string[];
        /**
         * The name of the unique identifier in the primary key array
         */
        readonly primaryKeyUuid: string;
    }
    /**
     * A type alias for a data source for a JSON data model.
     *
     * A data source is an array of JSON object records which represent
     * the rows of the table. The keys of the records correspond to the
     * field names of the columns.
     */
    type DataSource = ReadonlyArray<ReadonlyJSONObject>;
    /**
     * An options object for initializing a view.
     */
    interface IOptions {
        /**
         * The schema for the for the view.
         *
         * The schema should be treated as an immutable object.
         */
        schema: ISchema;
        /**
         * The data source for the view.
         *
         * The data model takes full ownership of the data source.
         */
        data: DataSource;
    }
}
