import { CellGroup, DataModel, MutableDataModel } from '@lumino/datagrid';
import { ReadonlyJSONObject, ReadonlyJSONValue } from '@lumino/coreutils';
import { ISignal, Signal } from '@lumino/signaling';
import { Transform } from './transform';
import { View } from './view';
import { TransformStateManager } from './transformStateManager';
/**
 * A view based data model implementation for in-memory JSON data.
 */
export declare class ViewBasedJSONModel extends MutableDataModel {
    /**
     * Create a data model with static JSON data.
     *
     * @param data - The dataset for initializing the data model.
     */
    constructor(data: ViewBasedJSONModel.IData);
    /**
     * Sets the dataset for this model.
     *
     * @param data - The data to be set on this data model
     */
    updateDataset(data: ViewBasedJSONModel.IData): void;
    /**
     * Updates the primary key map, which provides a mapping from primary key
     * value to row in the full dataset.
     */
    private _updatePrimaryKeyMap;
    areCellsMerged(cell1: number[], cell2: number[]): boolean;
    /**
     * Returns a list of [row, column] cells indices forming a merged cell group
     * @param row row index
     * @param column column index
     */
    getMergedSiblingCells(cell: number[]): any[];
    /**
     * Get the row count for a region in the data model.
     *
     * @param region - The row region of interest.
     *
     * @returns - The row count for the region.
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * Get the column count for a region in the data model.
     *
     * @param region - The column region of interest.
     *
     * @returns - The column count for the region.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * Get the group count for each region
     * @param region
     * @returns
     */
    groupCount(region: DataModel.RowRegion): number;
    /**
     * Specify merged cell groups for each region
     * @param region
     * @param groupIndex
     * @returns
     */
    group(region: DataModel.CellRegion, groupIndex: number): CellGroup | null;
    /**
     * Get the metadata for a column in the data model.
     *
     * @param region - The cell region of interest.
     *
     * @param column - The index of the column of interest.
     *
     * @returns The metadata for the column.
     */
    metadata(region: DataModel.CellRegion, row: number, column: number): DataModel.Metadata;
    /**
     * Get the data value for a cell in the data model.
     *
     * @param region - The cell region of interest.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param column - The column index of the cell of interest.
     *
     * @param returns - The data value for the specified cell.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * Updates the cell value of the currently displayed View.
     *
     * @param region - The cell region of interest.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param column - The column index of the cell of interest.
     *
     * @param value - The new value to update the indicated cell with.
     *
     */
    setData(region: DataModel.CellRegion, row: number, column: number, value: any): boolean;
    /**
     * Updates the cell value of the currently displayed View.
     *
     * @param region - The cell region of interest.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param column - The column index of the cell of interest.
     *
     * @param value - The new value to update the indicated cell with.
     *
     */
    setModelData(region: DataModel.CellRegion, row: number, column: number, value: any): boolean;
    /**
     * Updates the row value of the currently displayed View.
     *
     * @param row - The row index of the cell of interest.
     *
     * @param value - The new value to update the indicated row with.
     *
     */
    setRowData(row: number, value: any): boolean;
    columnNameToIndex(name: string): number;
    columnIndexToName(index: number, region: DataModel.ColumnRegion): string;
    columnNameToRegion(name: string): DataModel.ColumnRegion;
    /**
     * Get the current View for the model.
     */
    protected get currentView(): View;
    /**
     * Sets the provided View as the current View, then emits a changed signal.
     */
    protected set currentView(view: View);
    /**
     * Add a new transform to the currently active transforms.
     *
     * @param transform - The transform to be added.
     */
    addTransform(transform: Transform.TransformSpec): void;
    /**
     * Removes the provided transformation from the active state.
     *
     * @param columnIndex - The index of the column state to be removed.
     *
     * @param transformType - The type of the transform to be removed from state.
     */
    removeTransform(columnIndex: number, transformType: string): void;
    /**
     * Apply an array of transformations to the dataset and update the current
     * View. The provided transforms will replace any existing ones.
     *
     * @param transforms - Array of transformations to apply to the dataset
     */
    replaceTransforms(transforms: Transform.TransformSpec[]): void;
    /**
     * Removes all active transforms.
     */
    clearTransforms(): void;
    /**
     * Returns the transform metadata for the provided column.
     *
     * @param columnIndex - The column index of the metadata to be retrieved.
     */
    transformMetadata(columnIndex: number): TransformStateManager.IColumn | undefined;
    /**
     * Returns a Promise that resolves to an array of unique values contained in
     * the provided column index.
     *
     * @param region - The CellRegion to retrieve unique values for.
     * @param columnIndex - The index to retrieve unique values for.
     */
    uniqueValues(region: DataModel.CellRegion, columnIndex: number): Promise<any[]>;
    /**
     * Returns a Promise that resolves to an array of unique values contained in
     * the provided column index after all transforms have been applied.
     *
     * @param region - The CellRegion to retrieve unique values for.
     * @param columnIndex - The index to retrieve unique values for.
     */
    uniqueValuesVisible(region: DataModel.CellRegion, columnIndex: number): Promise<any[]>;
    get transformStateChanged(): ISignal<this, TransformStateManager.IEvent>;
    /**
     * Returns the active transforms for the current model
     */
    get activeTransforms(): Transform.TransformSpec[];
    getFilterTransform(columnIndex: number): Transform.TransformSpec | undefined;
    /**
     * Updates the indicated value in the dataset.
     *
     * Note: provided row/column values should correspond to the currently
     * active View, not the full dataset.
     * Note: Currently, only updating `body` cells is supported.
     *
     * @param options - The options for this method.
     */
    getDatasetRowFromView(region: DataModel.CellRegion, row: number): number;
    /**
     * Updates a value in the full dataset of the model.
     *
     * @param options - The options for this function.
     */
    updateCellValue(options: ViewBasedJSONModel.IUpdateCellValuesOptions): void;
    /**
     * Updates a row in the full dataset of the model.
     *
     * @param options - The options for this function.
     */
    updateRowValue(options: ViewBasedJSONModel.IUpdateRowValuesOptions): void;
    /**
     * A signal emitted when the data model has changes to sync to the kernel.
     */
    get dataSync(): Signal<this, ViewBasedJSONModel.IDataSyncEvent>;
    /**
     * Returns the current full dataset.
     */
    get dataset(): ViewBasedJSONModel.IData;
    /**
     * Returns the index in the schema that relates to the index by region.
     *
     * @param region - The `CellRegion` of interest.
     *
     * @param index - The column index to look up.
     */
    getSchemaIndex(region: DataModel.CellRegion, index: number): number;
    /**
     * Deep copies data object and mutates it before
     * returning a ViewBasedJSONModel of the data.
     * ts-ignores are added since the properties to be mutated
     * are readonly
     *
     * @param data - Data passed in to be transformed
     *
     */
    static fromJsonData(data: ViewBasedJSONModel.IData): ViewBasedJSONModel;
    private _currentView;
    private _transformSignal;
    private _dataSyncSignal;
    private _primaryKeyMap;
    protected _dataset: ViewBasedJSONModel.IData;
    protected readonly _transformState: TransformStateManager;
    private _mergedColumnCellLocations;
    private _rowCellGroups;
    private _columnCellGroups;
}
/**
 * The namespace for the `ViewBasedJSONModel` class statics.
 */
export declare namespace ViewBasedJSONModel {
    /**
     * An object which describes a column of data in the model.
     *
     * #### Notes
     * This is based on the JSON Table Schema specification:
     * https://specs.frictionlessdata.io/table-schema/
     */
    interface IField {
        /**
         * The name of the column.
         *
         * This is used as the key to extract a value from a data record.
         */
        readonly name: string;
        /**
         * The type of data held in the column.
         */
        readonly type: string;
        /**
         * An array of the column labels per header row.
         */
        readonly rows: any[];
    }
    /**
     * An object when specifies the schema for a data model.
     *
     * #### Notes
     * This is based on the JSON Table Schema specification:
     * https://specs.frictionlessdata.io/table-schema/
     */
    interface ISchema {
        /**
         * The fields which describe the data model columns.
         *
         * Primary key fields are rendered as row header columns.
         */
        readonly fields: IField[];
        /**
         * The field names which act as primary keys.
         *
         * Primary key fields are rendered as row header columns.
         */
        readonly primaryKey: string[];
        readonly primaryKeyUuid: string;
    }
    interface IUpdateCellValuesOptions {
        /**
         * The `CellRegion` of the cell to be updated.
         */
        region: DataModel.CellRegion;
        /**
         * The index of the target row in the current view.
         */
        column: number;
        /**
         * The index of the target row in the current view.
         */
        row: number;
        /**
         * The new value to replace the old one.
         */
        value: any;
        /**
         * The flag to trigger full data sync with backend.
         */
        syncData?: boolean;
    }
    interface IUpdateRowValuesOptions {
        /**
         * The index of the target row in the current view.
         */
        row: number;
        /**
         * The new value to replace the old one.
         */
        value: any[];
        /**
         * The flag to trigger full data sync with backend.
         */
        syncData?: boolean;
    }
    /**
     * A type alias for a data source for a JSON data model.
     *
     * A data source is an array of JSON object records which represent
     * the rows of the table. The keys of the records correspond to the
     * field names of the columns.
     */
    type DataSource = ReadonlyArray<ReadonlyJSONObject>;
    /**
     * An options object for initializing the data model.
     */
    interface IData {
        /**
         * The schema for the for the data model.
         *
         * The schema should be treated as an immutable object.
         */
        schema: ISchema;
        /**
         * The data source for the data model.
         *
         * The data model takes full ownership of the data source.
         */
        data: DataSource;
    }
    type IDataSyncEvent = ISyncCell | ISyncRowIndices | ICellEditEvent;
    /**
     * An event that indicates a needed change to the kernel-side dataset.
     */
    interface ISyncCell {
        /**
         * The discriminated type of the args object.
         */
        type: 'cell-updated';
    }
    interface ISyncRowIndices {
        /**
         * The discriminated type of the args object.
         */
        type: 'row-indices-updated';
        /**
         * An list of the rows in the untransformed dataset that are currently
         * represented in the `View`.
         */
        indices: number[];
    }
    interface ICellEditEvent {
        /**
         * The discriminated type of the args object.
         */
        type: 'cell-edit-event';
        /**
         * The CellRegion associated with this change.
         */
        region: DataModel.CellRegion;
        /**
         * The row number associated with this change.
         */
        row: number;
        /**
         * The column index associated with this change.
         */
        columnIndex: number;
        /**
         * The new data value
         */
        value: ReadonlyJSONValue;
    }
}
